SystemOrganization addCategory: #SimplePersistence!
SystemOrganization addCategory: #'SimplePersistence-Tests'!

TestCase subclass: #SimplePersistenceTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimplePersistence-Tests'!

!SimplePersistenceTests methodsFor: 'as yet unclassified' stamp: 'SeanDeNigris 5/29/2011 13:19'!
testSaveObjects

	| db numberToStore stringToStore |
	db := SimplePersistenceTestDatabase.
	numberToStore := 1.
	stringToStore := 'you can use any object, not just numbers and strings!!'.
	
	db store: numberToStore and: stringToStore.
	db saveRepository.
	
	db reset.
	db restoreLastBackup.
	
	self assert: (db number = numberToStore).
	self assert: (db string = stringToStore).! !

!SimplePersistenceTests methodsFor: 'as yet unclassified' stamp: 'SeanDeNigris 5/29/2011 13:21'!
testSaveObjectsInBackground

	| db numberToStore stringToStore |
	db := SimplePersistenceTestDatabase.
	numberToStore := 1.
	stringToStore := 'you can use any object, not just numbers and strings!!'.
	
	db store: numberToStore and: stringToStore.
	db takeSnapshot.
	
	db reset.
	db restoreLastBackup.
	
	self assert: (db number = numberToStore).
	self assert: (db string = stringToStore).! !

!DataStream class methodsFor: '*SimplePersistence' stamp: 'SeanDeNigris 5/29/2011 12:54'!
detectFile: aBlock do: anotherBlock

	^aBlock value
		ifNil: [nil]
      	ifNotNil: [:file| [anotherBlock value: file] ensure: [file close]]! !

!DataStream class methodsFor: '*SimplePersistence' stamp: 'SeanDeNigris 5/29/2011 12:54'!
newFileNamed: fileName do: aBlock
	"Returns the result of aBlock."
	
	^ self detectFile: [ self newFileNamed: fileName ] do: aBlock! !

!DataStream class methodsFor: '*SimplePersistence' stamp: 'SeanDeNigris 5/29/2011 12:53'!
readOnlyFileNamed: aName do: aBlock 
    | file |
    file := self readOnlyFileNamed: aName.
    ^ file 
        ifNil: [ nil ]
        ifNotNil: [ [ aBlock value: file ] ensure: [ file close ] ]! !

Object subclass: #SPFileDatabase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimplePersistence'!

SPFileDatabase class
	instanceVariableNames: 'lock'!

!SPFileDatabase commentStamp: 'SeanDeNigris 5/29/2011 13:36' prior: 0!
Based on the idea that:
* most applications will not have to scale (i.e. become the next Twitter)
* simply saving the image is slow and error-prone

For the full motivation, see Ramon Leon's blog post at http://onsmalltalk.com/simple-image-based-persistence-in-squeak/.

To give your application persistence:
	1. Subclass SMFileDatabase
	2. On the subclass, implement:
		a. class>>repositories (see method comment).
		b. class>>restoreRepositories: (see method comment).
		
That's it!! Now, whenever you want to save, call class>>saveRepository or class>>takeSnapshot (background save).

To customize:
* Number of backups kept: override class>>defaultHistoryCount
			!

SPFileDatabase class
	instanceVariableNames: 'lock'!

!SPFileDatabase class methodsFor: 'private' stamp: 'rjl 12/4/2007 17:57'!
backupDirectory
	^ (FileDirectory default directoryNamed: self name) assureExistence.
! !

!SPFileDatabase class methodsFor: 'private' stamp: 'SeanDeNigris 5/29/2011 13:22'!
defaultHistoryCount
	"How many backups to keep."
	^ 50! !

!SPFileDatabase class methodsFor: 'startup/shutdown' stamp: 'rjl 12/4/2007 18:15'!
enablePersistence
	Smalltalk addToStartUpList: self.
	Smalltalk addToShutDownList: self! !

!SPFileDatabase class methodsFor: 'private' stamp: 'rjl 1/14/2008 08:50'!
lastBackup
	| lastBackup |
	lastBackup := self lastBackupFile.
	lastBackup ifNil: [ ^ nil ].
	^ ReferenceStream 
		readOnlyFileNamed: (self backupDirectory fullNameFor: lastBackup)
		do: [ : f | f next ]! !

!SPFileDatabase class methodsFor: 'private' stamp: 'rjl 12/4/2007 18:28'!
lastBackupFile
	^ self backupDirectory fileNames 
		detectMax: [:each | each name asInteger]! !

!SPFileDatabase class methodsFor: 'override in subclasses' stamp: 'SeanDeNigris 5/29/2011 13:27'!
repositories
	"Return the root object that needs serialized (e.g. an array containing the root collection of each domain class)."
	self subclassResponsibility! !

!SPFileDatabase class methodsFor: 'restoring' stamp: 'rjl 12/11/2007 12:17'!
restoreLastBackup
	self lastBackup ifNotNilDo: [ : backup | self restoreRepositories: backup ]! !

!SPFileDatabase class methodsFor: 'override in subclasses' stamp: 'SeanDeNigris 5/29/2011 13:29'!
restoreRepositories: someRepositories
	"Put them back  wherever they belong in the image for the application to use them."
	self subclassResponsibility! !

!SPFileDatabase class methodsFor: 'saving' stamp: 'rjl 1/14/2008 08:52'!
saveRepository
	| version |
	lock ifNil: [ lock := Semaphore forMutualExclusion ].
	lock critical: 
		[ self trimBackups.
		version := self lastBackupFile 
			ifNil: [ 1 ]
			ifNotNil: [ self lastBackupFile name asInteger + 1 ].
		ReferenceStream 
			newFileNamed: (self backupDirectory fullPathFor: self name) , '.' , version asString
			do: 
				[ : f | 
				f
					nextPut: self repositories ;
					flush ] ]! !

!SPFileDatabase class methodsFor: 'startup/shutdown' stamp: 'rjl 12/4/2007 18:19'!
shutDown: isGoingDown 
	isGoingDown ifTrue: [ self saveRepository ]! !

!SPFileDatabase class methodsFor: 'startup/shutdown' stamp: 'rjl 12/4/2007 18:24'!
startUp: isComingUp 
	isComingUp ifTrue: [ self restoreLastBackup ]! !

!SPFileDatabase class methodsFor: 'restoring' stamp: 'SeanDeNigris 5/29/2011 13:26'!
takeSnapshot
	"Motivation (vs. #restoreLastBackup): if you want a little extra snappiness and you're not worried about making the user wait for the flush to disk"
	[self saveRepository] forkAt: Processor userBackgroundPriority
		named: 'snapshot: ' , self class name! !

!SPFileDatabase class methodsFor: 'private' stamp: 'rjl 12/11/2007 12:19'!
trimBackups
	| entries versionsToKeep |
	versionsToKeep := self defaultHistoryCount.
	entries := self backupDirectory entries.
	entries size < versionsToKeep ifTrue: [ ^ self ].
	((entries sortBy: [ : a : b | a first asInteger < b first asInteger ]) allButLast: versionsToKeep) 
		do: [ : entry | self backupDirectory deleteFileNamed: entry first ]! !

SPFileDatabase subclass: #SimplePersistenceTestDatabase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimplePersistence-Tests'!

SimplePersistenceTestDatabase class
	instanceVariableNames: 'number string'!

SimplePersistenceTestDatabase class
	instanceVariableNames: 'number string'!

!SimplePersistenceTestDatabase class methodsFor: 'accessing' stamp: 'SeanDeNigris 5/29/2011 13:19'!
number

	^ number.! !

!SimplePersistenceTestDatabase class methodsFor: 'as yet unclassified' stamp: 'SeanDeNigris 5/29/2011 13:11'!
repositories

	^ { number. string }! !

!SimplePersistenceTestDatabase class methodsFor: 'as yet unclassified' stamp: 'SeanDeNigris 5/29/2011 13:13'!
reset

	number := string := nil.! !

!SimplePersistenceTestDatabase class methodsFor: 'as yet unclassified' stamp: 'SeanDeNigris 5/29/2011 13:16'!
restoreRepositories: someRepositories

	number := someRepositories at: 1.
	string := someRepositories at: 2.! !

!SimplePersistenceTestDatabase class methodsFor: 'as yet unclassified' stamp: 'SeanDeNigris 5/29/2011 13:12'!
store: aNumber and: aString

	number := aNumber.
	string := aString.! !

!SimplePersistenceTestDatabase class methodsFor: 'accessing' stamp: 'SeanDeNigris 5/29/2011 13:19'!
string

	^ string.! !
